@@ -1,5 +1,6 @@
 ### FuzzyR - Utilities
 
+
 ## @export
 meshgrid <- function(a,b) {
   list(x=outer(b*0, a, FUN="+"), y=outer(b, a*0, FUN="+"))
@@ -99,6 +100,7 @@ init.params.gbell <- function(x, n=2) {
 	x.range <- range(x)
 
     a <- diff(x.range) / 5
+    if(a == 0) a = 1
     b <- 1
     c <- seq(x.range[1], x.range[2], length.out=n)
 
@@ -121,6 +123,7 @@ init.params.it2gbell <- function(x, n=2) {
 	x.range <- range(x)
 
     a.lower <- diff(x.range) / 5
+    if(a.lower == 0) a.lower = 1
     a.upper = a.lower * 1.2
     b = 1
     c <- seq(x.range[1], x.range[2], length.out=n)
@@ -184,7 +187,8 @@ rspe <- function(f, y, ref=0) {
 #' fuzzyr.accuracy(f, y)
 #' @author Chao Chen
 #' @references
-#' A new accuracy measure based on bounded relative error for time series forecasting \url{http://dx.doi.org/10.1371/journal.pone.0174202}
+#' [1] C. Chen, J. Twycross, and J. M. Garibaldi, “A new accuracy measure based on bounded relative error for time series forecasting,” PLOS ONE, vol. 12, no. 3, pp. 1–23, 2017. \cr
+#' \url{http://dx.doi.org/10.1371/journal.pone.0174202}
 #' @export
 
 fuzzyr.accuracy <- function(f, y, f.ref=0, scale.mase=NULL) {
